package com.hncy.doc.spring.boot.starter.aggregate;

import org.springframework.cloud.netflix.zuul.filters.ZuulProperties;
import springfox.documentation.swagger.web.SwaggerResource;
import springfox.documentation.swagger.web.SwaggerResourcesProvider;

import java.util.List;

/**
 * @Author: ying.dong
 * @Date: 2021/4/25 14:58
 */
public class AggregateSwaggerResourceProvider implements SwaggerResourcesProvider {

    private final ZuulProperties zuulProperties;
    private final SwaggerResourcesProvider delegate;

    //DOC_V3 = "/v3/api-docs";
    private final String docPath;
    private final String swaggerVersion;

    public AggregateSwaggerResourceProvider(ZuulProperties zuulProperties,
                                            SwaggerResourcesProvider delegate,
                                            String docPath, String swaggerVersion) {
        this.zuulProperties = zuulProperties;
        this.delegate = delegate;
        this.docPath = docPath;
        this.swaggerVersion = swaggerVersion;
    }

    @Override
    public List<SwaggerResource> get() {
        List<SwaggerResource> resources = delegate.get();
        zuulProperties.getRoutes().values()
                .forEach(route -> resources.add(resource(route.getServiceId(), route.getId())));
        return resources;
    }

    private SwaggerResource resource(String swaggerGroup, String baseUrl) {
        String serviceDocLocation = "/" + baseUrl + docPath;
        SwaggerResource swaggerResource = new SwaggerResource();
        swaggerResource.setName(swaggerGroup);
        swaggerResource.setLocation(serviceDocLocation);
        swaggerResource.setSwaggerVersion(swaggerVersion);
        return swaggerResource;
    }
}
