package com.hncy.doc.spring.boot.starter.aggregate;

import org.springframework.http.HttpRequest;
import org.springframework.http.server.ServletServerHttpRequest;
import org.springframework.web.util.UriComponents;
import org.springframework.web.util.UriComponentsBuilder;

import javax.servlet.http.HttpServletRequest;
import java.net.URI;
import java.util.Enumeration;

/**
 * @Author: ying.dong
 * @Date: 2021/4/25 14:19
 */
public class ForwardHeaderExtractor {

    private static final String FORWARD_PREFIX = "X-Forwarded-Prefix";

    private final UriComponents uriComponents;

    private final String forwardPrefix;

    public ForwardHeaderExtractor(HttpServletRequest request) {
        HttpRequest httpRequest = new ServletServerHttpRequest(request);
        this.uriComponents = UriComponentsBuilder.fromHttpRequest(httpRequest).build();
        this.forwardPrefix = initForwardedPrefix(request);
    }

    private String initForwardedPrefix(HttpServletRequest request) {
        String result = null;
        Enumeration<String> names = request.getHeaderNames();
        while (names.hasMoreElements()) {
            String name = names.nextElement();
            if (FORWARD_PREFIX.equalsIgnoreCase(name)) {
                result = request.getHeader(name);
            }
        }
        if (result != null) {
            while (result.endsWith("/")) {
                result = result.substring(0, result.length() - 1);
            }
        }
        return result;
    }

    private String getUnnormalizedUrl() {
        return String.format("%s://%s:%s/%s",
                uriComponents.getScheme(),
                uriComponents.getHost(),
                uriComponents.getPort(),
                forwardPrefix == null ? "/" : forwardPrefix);
    }

    public String getUrl() {
        return URI.create(getUnnormalizedUrl()).normalize().toString();
    }

    public String getForwardPrefix() {
        return forwardPrefix;
    }
}
