package com.hncy.doc.spring.boot.starter.aggregate;

import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.PathItem;
import io.swagger.v3.oas.models.Paths;
import io.swagger.v3.oas.models.servers.Server;
import org.springframework.util.StringUtils;
import springfox.documentation.oas.web.OpenApiTransformationContext;
import springfox.documentation.oas.web.WebMvcOpenApiTransformationFilter;
import springfox.documentation.spi.DocumentationType;

import javax.servlet.http.HttpServletRequest;
import java.util.Collections;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 网关聚合doc时修改server的url，不然在前端调试时会404
 * @Author: ying.dong
 * @Date: 2021/4/25 11:45
 */
public class ProxyWebMvcOpenApiTransformationFilter implements WebMvcOpenApiTransformationFilter {

    @Override
    public OpenAPI transform(OpenApiTransformationContext<HttpServletRequest> context) {
        OpenAPI openApi = context.getSpecification();
        context.request().ifPresent(servletRequest -> {
            ForwardHeaderExtractor extractor = new ForwardHeaderExtractor(servletRequest);
            Server server = new Server()
                    .url(extractor.getUrl())
                    .description("Inferred Url");
            openApi.servers(Collections.singletonList(server));

            //rebuild paths (forwardPrefix + path)
            if (!StringUtils.isEmpty(extractor.getForwardPrefix())) {
                Map<String, PathItem> pathItemMap = openApi.getPaths().entrySet().stream()
                        .collect(Collectors.toMap(it -> extractor.getForwardPrefix() + it.getKey(), Map.Entry::getValue));
                Paths paths = new Paths();
                paths.putAll(pathItemMap);
                openApi.paths(paths);
            }

        });
        return openApi;
    }

    @Override
    public boolean supports(DocumentationType documentationType) {
        return documentationType == DocumentationType.OAS_30;
    }

}
