package com.hncy.exception.handler.spring.boot.starter.config;

import brave.Tracer;
import com.hncy.exception.handler.spring.boot.starter.common.AlarmEventPublisher;
import com.hncy.exception.handler.spring.boot.starter.common.ConfigProperties;
import com.hncy.exception.handler.spring.boot.starter.core.alarm.AlarmReporter;
import com.hncy.exception.handler.spring.boot.starter.core.translator.translators.alarm.DefaultAlarmTranslator;
import com.hncy.exception.handler.spring.boot.starter.core.translator.translators.alarm.TracingAlarmTranslator;
import com.hncy.exception.handler.spring.boot.starter.handler.AlarmEventListener;
import com.hncy.notifier.spring.boot.starter.core.Notifier;
import org.springframework.boot.autoconfigure.condition.*;
import org.springframework.cloud.sleuth.autoconfig.TraceAutoConfiguration;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @Author: ying.dong
 * @Date: 2021/4/19 18:43
 */
@Configuration
@ConditionalOnProperty(prefix = ConfigProperties.PREFIX, name = "alarm",
        havingValue = "true")
public class AlarmConfig {

    @Bean
    public AlarmEventPublisher alarmEventPublisher() {
        return new AlarmEventPublisher();
    }

    @Bean
    public AlarmEventListener alarmEventListener(AlarmReporter alarmReporter) {
        return new AlarmEventListener(alarmReporter);
    }

    @Bean
    @ConditionalOnMissingBean(AlarmReporter.class)
    public AlarmReporter defaultAlarmReporter(Notifier notifier) {
        return new DefaultAlarmTranslator(notifier);
    }


    @Configuration
    @ConditionalOnClass({Tracer.class, TraceAutoConfiguration.class})
    static class TracingConfiguration {

        @Bean
        @ConditionalOnMissingBean(AlarmReporter.class)
        public AlarmReporter tracingAlarmReporter(Notifier notifier, Tracer tracer) {
            return new TracingAlarmTranslator(notifier, tracer);
        }
    }
}
