package com.hncy.exception.handler.spring.boot.starter.config;

import com.hncy.common.exception.common.MessageSourceHolder;
import com.hncy.exception.handler.spring.boot.starter.common.ConfigProperties;
import com.hncy.exception.handler.spring.boot.starter.core.translator.ExceptionTranslationExecutor;
import com.hncy.exception.handler.spring.boot.starter.core.translator.base.ExceptionTranslator;
import com.hncy.exception.handler.spring.boot.starter.handler.GlobalExceptionController;
import com.hncy.exception.handler.spring.boot.starter.handler.GlobalExceptionHandler;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnWebApplication;
import org.springframework.boot.autoconfigure.web.ServerProperties;
import org.springframework.boot.web.servlet.error.ErrorAttributes;
import org.springframework.context.annotation.Bean;
import org.springframework.context.support.ResourceBundleMessageSource;

import org.springframework.web.servlet.DispatcherServlet;

/**
 * @Author: ying.dong
 * @Date: 2021/4/19 16:30
 */
public class ExceptionConfig {

    private final ConfigProperties properties;

    public ExceptionConfig(ConfigProperties configProperties) {
        this.properties = configProperties;
    }

    @Bean(name = "messageSource")
    public ResourceBundleMessageSource messageSource() {
        ResourceBundleMessageSource source = new ResourceBundleMessageSource();
        source.setBasenames(properties.getMessagePath());
        source.setUseCodeAsDefaultMessage(true);
        source.setDefaultEncoding("UTF-8");

        MessageSourceHolder.set(source);
        return source;
    }

    @ConditionalOnMissingBean
    @Bean
    public ExceptionTranslator exceptionTranslator() {
        return new ExceptionTranslationExecutor();
    }


    @ConditionalOnWebApplication(type = ConditionalOnWebApplication.Type.SERVLET)
    @ConditionalOnClass(DispatcherServlet.class)
    @Bean
    public GlobalExceptionController globalExceptionController(ExceptionTranslator exceptionTranslator, ErrorAttributes errorAttributes, ServerProperties serverProperties) {
        return new GlobalExceptionController(exceptionTranslator, errorAttributes, serverProperties.getError());
    }

    @ConditionalOnWebApplication(type = ConditionalOnWebApplication.Type.SERVLET)
    @ConditionalOnClass(DispatcherServlet.class)
    @Bean
    public GlobalExceptionHandler globalExceptionHandler(ExceptionTranslator exceptionTranslator) {
        return new GlobalExceptionHandler(exceptionTranslator);
    }
}
