package com.hncy.exception.handler.spring.boot.starter.core.translator.translators.alarm;

import cn.hutool.core.exceptions.ExceptionUtil;
import com.hncy.common.exception.common.GlobalErrorConstants;
import com.hncy.common.exception.custom.AlarmBusinessException;
import com.hncy.common.exception.custom.LocalAlarmBusinessException;
import com.hncy.exception.handler.spring.boot.starter.common.*;
import com.hncy.exception.handler.spring.boot.starter.core.alarm.AlarmReporter;
import com.hncy.exception.handler.spring.boot.starter.core.translator.base.AbstractExceptionTranslator;
import com.hncy.notifier.spring.boot.starter.core.Notifier;
import com.hncy.notifier.spring.boot.starter.core.NotifierResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.http.HttpStatus;
import org.springframework.scheduling.annotation.Async;

/**
 * @Author: ying.dong
 * @Date: 2021/4/19 16:24
 */
@Slf4j
public class DefaultAlarmTranslator extends AbstractExceptionTranslator implements AlarmReporter, ApplicationContextAware {

    private final Notifier notifier;

    private ApplicationContext applicationContext;

    public DefaultAlarmTranslator(Notifier notifier) {
        this.notifier = notifier;
    }

    @Override
    protected ApiResult doTranslate(Throwable throwable) {
        int code = GlobalErrorConstants.INTERNAL_SERVER_ERROR.getCode();
        if (throwable instanceof LocalAlarmBusinessException) {
            code = ((LocalAlarmBusinessException) throwable).getCode();
        }
        report(throwable);
        return ApiResultBuilder.build(code, throwable.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
    }

    @Override
    public boolean support(Throwable throwable) {
        return throwable instanceof AlarmBusinessException;
    }


    @Async
    @Override
    public void report(Throwable throwable) {
        String stacktrace = doResolveContent() + doResolveExceptionContent(throwable);
        NotifierResult notify = notifier.notify(stacktrace);
        log.info("{} reported result: {}", throwable.getClass().getName(), notify.isSuccess());
    }

    @Async
    @Override
    public void report(String content) {
        String alarmContent = doResolveContent() + content;
        NotifierResult notify = notifier.notify(alarmContent);
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }

    protected String doResolveContent() {
        String applicationName = applicationContext.getEnvironment().getProperty("spring.application.name");
        return "[" + applicationName + "]  ";
    }

    protected String doResolveExceptionContent(Throwable throwable) {
        Throwable rootCause = ExceptionUtil.getRootCause(throwable);
        return ExceptionUtil.stacktraceToString(rootCause);
    }
}
