package com.hncy.feign.spring.boot.starter.form;

import feign.Feign;
import feign.Retryer;
import feign.codec.Encoder;
import feign.form.FormEncoder;
import feign.querymap.BeanQueryMapEncoder;
import org.springframework.beans.factory.ObjectFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnWebApplication;
import org.springframework.boot.autoconfigure.http.HttpMessageConverters;
import org.springframework.cloud.openfeign.support.SpringEncoder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

/**
 * @Author: ying.dong
 * @Date: 2021/5/12 09:49
 */
@Configuration
@ConditionalOnWebApplication(type = ConditionalOnWebApplication.Type.SERVLET)
public class FeignFormConfiguration {

    @Autowired
    private ObjectFactory<HttpMessageConverters> messageConverters;

    @Bean
    public Encoder feignFormEncoder() {
        return new FormEncoder(new SpringEncoder(messageConverters));
    }

    @Bean
    @ConditionalOnMissingBean
    public Retryer feignRetryer() {
        return Retryer.NEVER_RETRY;
    }

    /**
     * 默认使用FieldQueryMapEncoder，该encoder不支持继承的父类属性
     * @param retryer
     * @return
     */
    @Bean
    @Primary
    public Feign.Builder feignBuilder(Retryer retryer) {
        return Feign.builder().retryer(retryer).queryMapEncoder(new BeanQueryMapEncoder());
    }
}
