package com.hncy.file.manage.spring.boot.starter.oss;

import com.hncy.file.manage.spring.boot.starter.oss.common.OSSClientFactory;
import com.hncy.file.manage.spring.boot.starter.oss.common.OSSClientRegistry;
import com.hncy.file.manage.spring.boot.starter.oss.constant.OSSConstant;
import com.hncy.file.manage.spring.boot.starter.oss.manager.OSSManager;
import com.hncy.file.manage.spring.boot.starter.oss.manager.OSSPubBucketManager;
import com.hncy.file.manage.spring.boot.starter.oss.manager.OSSServerManager;
import com.hncy.file.manage.spring.boot.starter.oss.manager.OSSSignedManager;
import com.hncy.file.manage.spring.boot.starter.oss.properties.OSSProperties;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

/**
 * @Author: ying.dong
 * @Date: 2021/5/22 14:48
 */
@Configuration
@EnableConfigurationProperties(OSSProperties.class)
@ConditionalOnProperty(
        value = "config.file.manage.oss.enabled",
        havingValue = "true",
        matchIfMissing = true
)
public class OSSAutoConfiguration {

    @Bean
    @ConditionalOnMissingBean
    OSSClientFactory ossClientFactory() {
        return new OSSClientFactory();
    }

    @ConditionalOnMissingBean
    @Bean
    OSSClientRegistry ossClientRegistry(OSSProperties ossProperties, OSSClientFactory ossClientFactory) {
        return new OSSClientRegistry(ossProperties, ossClientFactory);
    }

    @Primary
    @Bean(name = OSSConstant.BeanName.SIGNED)
    OSSManager OSSSignedManager(OSSClientRegistry ossClientRegistry) {
        return new OSSSignedManager(ossClientRegistry);
    }

    @Bean(name = OSSConstant.BeanName.SERVER)
    OSSManager OSSServerManager(OSSClientRegistry ossClientRegistry) {
        return new OSSServerManager(ossClientRegistry);
    }

    @Bean(name = OSSConstant.BeanName.PUBLIC)
    OSSManager OSSPubBucketManager(OSSClientRegistry ossClientRegistry) {
        return new OSSPubBucketManager(ossClientRegistry);
    }
}
