package com.hncy.message.spring.boot.starter;

import brave.spring.rabbit.SpringRabbitTracing;
import com.hncy.message.spring.boot.starter.common.ExchangeTypeEnum;
import com.hncy.message.spring.boot.starter.consumer.DynamicMessageListenerContainerFactory;
import com.hncy.message.spring.boot.starter.consumer.initializer.DefaultDynamicMessageListenerInitializer;
import com.hncy.message.spring.boot.starter.consumer.initializer.DynamicMessageListenerInitializer;
import com.hncy.message.spring.boot.starter.metadata.resolver.DelegatingMessageMetadataResolver;
import com.hncy.message.spring.boot.starter.metadata.resolver.DirectMessageMetadataResolver;
import com.hncy.message.spring.boot.starter.metadata.resolver.FanoutMessageMetadataResolver;
import com.hncy.message.spring.boot.starter.metadata.resolver.MessageMetadataResolver;
import com.hncy.message.spring.boot.starter.producer.DelegatingMessageDelivery;
import com.hncy.message.spring.boot.starter.producer.DirectMessageDelivery;
import com.hncy.message.spring.boot.starter.producer.FanoutMessageDelivery;
import com.hncy.message.spring.boot.starter.producer.MessageDelivery;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author: ying.dong
 * @Date: 2021/6/3 13:45
 */
@Configuration
public class MessageAutoConfiguration {

    @ConditionalOnClass(SpringRabbitTracing.class)
    static class TracingConfiguration {

        @Bean
        @ConditionalOnBean(SpringRabbitTracing.class)
        @Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE, proxyMode = ScopedProxyMode.TARGET_CLASS)
        public RabbitTemplate rabbitTemplate(ConnectionFactory connectionFactory, SpringRabbitTracing springRabbitTracing) {
            return springRabbitTracing.newRabbitTemplate(connectionFactory);
        }

        @Bean
        @ConditionalOnBean(SpringRabbitTracing.class)
        @ConditionalOnMissingBean
        DynamicMessageListenerContainerFactory tracingFactory(ConnectionFactory connectionFactory, SpringRabbitTracing springRabbitTracing) {
            DynamicMessageListenerContainerFactory dynamicMessageListenerContainerFactory = new DynamicMessageListenerContainerFactory(connectionFactory);
            return (DynamicMessageListenerContainerFactory) springRabbitTracing.decorateSimpleRabbitListenerContainerFactory(dynamicMessageListenerContainerFactory);
        }
    }


    @Bean
    @ConditionalOnMissingBean
    public RabbitAdmin amqpAdmin(ConnectionFactory connectionFactory) {
        return new RabbitAdmin(connectionFactory);
    }

    @Bean
    @Scope(value = ConfigurableBeanFactory.SCOPE_PROTOTYPE, proxyMode = ScopedProxyMode.TARGET_CLASS)
    public RabbitTemplate rabbitTemplate(ConnectionFactory connectionFactory) {
        return new RabbitTemplate(connectionFactory);
    }

    @ConditionalOnMissingBean
    @Bean
    MessageMetadataResolver messageMetadataResolver(RabbitAdmin rabbitAdmin) {
        List<MessageMetadataResolver> metadataResolverList = new ArrayList<>();
        metadataResolverList.add(new FanoutMessageMetadataResolver(rabbitAdmin));
        metadataResolverList.add(new DirectMessageMetadataResolver(rabbitAdmin));
        return new DelegatingMessageMetadataResolver(metadataResolverList);
    }


    @Bean
    @ConditionalOnMissingBean
    MessageDelivery messageDelivery(MessageMetadataResolver messageMetadataResolver,
                                    RabbitTemplate rabbitTemplate) {
        Map<ExchangeTypeEnum, MessageDelivery> messageDeliveryMap = new HashMap<>();
        messageDeliveryMap.put(ExchangeTypeEnum.DIRECT, new DirectMessageDelivery(messageMetadataResolver, rabbitTemplate));
        messageDeliveryMap.put(ExchangeTypeEnum.FANOUT, new FanoutMessageDelivery(messageMetadataResolver, rabbitTemplate));
        return new DelegatingMessageDelivery(messageDeliveryMap, true);
    }

    @Bean
    @ConditionalOnMissingBean
    DynamicMessageListenerContainerFactory factory(ConnectionFactory connectionFactory) {
        return new DynamicMessageListenerContainerFactory(connectionFactory);
    }

    @Bean
    @ConditionalOnMissingBean
    DynamicMessageListenerInitializer dynamicMessageListenerInitializer() {
        return new DefaultDynamicMessageListenerInitializer();
    }

}
