package com.hncy.message.spring.boot.starter.metadata.resolver;

import com.hncy.message.spring.boot.starter.common.ExchangeTypeEnum;
import com.hncy.message.spring.boot.starter.metadata.FanoutMessageMetadata;
import com.hncy.message.spring.boot.starter.metadata.MessageMetadata;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.util.StringUtils;

/**
 * @Author: ying.dong
 * @Date: 2021/6/4 14:35
 */
public class FanoutMessageMetadataResolver extends AbstractMessageMetadataResolver {

    public FanoutMessageMetadataResolver(RabbitAdmin rabbitAdmin) {
        super(rabbitAdmin);
    }

    @Override
    public boolean isSupport(MessageMetadata messageMetadata) {
        return ExchangeTypeEnum.FANOUT.equals(messageMetadata.getExchangeType());
    }

    @Override
    public boolean doResolve(MessageMetadata messageMetadata) {
        FanoutMessageMetadata fanoutMessageMetadata = (FanoutMessageMetadata) messageMetadata;
        FanoutExchange exchange = new FanoutExchange(fanoutMessageMetadata.getExchange(), true, false, messageMetadata.getExchangeArgs());
        declareExchange(exchange);

        if (!StringUtils.isEmpty(fanoutMessageMetadata.getQueue())) {
            Queue queue = new Queue(fanoutMessageMetadata.getQueue(), true, false, false, messageMetadata.getQueueArgs());
            Binding binding = BindingBuilder.bind(queue).to(exchange);
            declareQueue(queue);
            declareBinding(binding);
        }
        return true;
    }
}
