package com.hncy.message.spring.boot.starter.producer;

import com.hncy.message.spring.boot.starter.common.ExchangeTypeEnum;
import com.hncy.message.spring.boot.starter.metadata.MessageMetadata;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Author: ying.dong
 * @Date: 2021/6/5 09:25
 */
@Slf4j
public class DelegatingMessageDelivery implements MessageDelivery {

    private final Map<ExchangeTypeEnum, MessageDelivery> messageDeliveryMap = new ConcurrentHashMap<>();

    private ApplicationContext applicationContext;

    /**
     * 是否触发初始化方法
     */
    private final boolean initialDelivery;

    public DelegatingMessageDelivery(@NonNull Map<ExchangeTypeEnum, MessageDelivery> messageDeliveryMap) {
        this(messageDeliveryMap, false);
    }

    public DelegatingMessageDelivery(@NonNull Map<ExchangeTypeEnum, MessageDelivery> messageDeliveryMap, @NonNull boolean initialDelivery) {
        this.messageDeliveryMap.putAll(messageDeliveryMap);
        this.initialDelivery = initialDelivery;
    }

    @Override
    public <T> boolean deliver(MessageMetadata messageMetadata, T data) {
        MessageDelivery messageDelivery = messageDeliveryMap.get(messageMetadata.getExchangeType());
        if (messageDelivery == null) {
            throw new RuntimeException("MessageDelivery with key [" + messageMetadata.getExchangeType() + "] not configured.");
        }
        return messageDelivery.deliver(messageMetadata, data);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        if (!initialDelivery) {
            return;
        }
        for (MessageDelivery messageDelivery: messageDeliveryMap.values()) {
            messageDelivery.setApplicationContext(applicationContext);
            messageDelivery.afterPropertiesSet();
        }
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }
}
