package com.hncy.message.spring.boot.starter.producer;

import com.hncy.message.spring.boot.starter.common.ExchangeTypeEnum;
import com.hncy.message.spring.boot.starter.metadata.DirectMessageMetadata;
import com.hncy.message.spring.boot.starter.metadata.MessageMetadata;
import com.hncy.message.spring.boot.starter.metadata.resolver.MessageMetadataResolver;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.lang.NonNull;

/**
 * @Author: ying.dong
 * @Date: 2021/6/5 09:20
 */
public class DirectMessageDelivery extends AbstractMessageDelivery<DirectMessageMetadata> {

    public DirectMessageDelivery(@NonNull MessageMetadataResolver messageMetadataResolver, @NonNull RabbitTemplate rabbitTemplate) {
        super(messageMetadataResolver, rabbitTemplate);
    }

    public DirectMessageDelivery(@NonNull MessageMetadataResolver messageMetadataResolver, @NonNull RabbitTemplate rabbitTemplate, @NonNull MessageNotDeliveredCallback messageNotDeliveredCallback, boolean autoDeclareMQ) {
        super(messageMetadataResolver, rabbitTemplate, messageNotDeliveredCallback, autoDeclareMQ);
    }

    @Override
    public boolean isSupport(MessageMetadata messageMetadata) {
        return ExchangeTypeEnum.DIRECT.equals(messageMetadata.getExchangeType());
    }

    @Override
    protected boolean doDeliver(DirectMessageMetadata messageMetadata, Message message) {
        rabbitTemplate.convertAndSend(messageMetadata.getExchange(), messageMetadata.getQueue(), message);
        return true;
    }
}
