package com.hncy.notifier.spring.boot.starter.mail;

import com.hncy.notifier.spring.boot.starter.core.AbstractNotifier;
import com.hncy.notifier.spring.boot.starter.core.DefaultNotifierResult;
import com.hncy.notifier.spring.boot.starter.core.NotifierResult;
import org.springframework.lang.NonNull;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import java.nio.charset.StandardCharsets;

/**
 * @Author: ying.dong
 * @Date: 2021/8/6 13:53
 */
public abstract class AbstractMailNotifier extends AbstractNotifier {

    protected final JavaMailSender javaMailSender;
    protected final MailProperties mailProperties;

    protected AbstractMailNotifier(JavaMailSender javaMailSender, MailProperties mailProperties) {
        this.javaMailSender = javaMailSender;
        this.mailProperties = mailProperties;
    }

    @Override
    protected NotifierResult doNotify(@NonNull String parameter) {
        try {
            MimeMessage mimeMessage = javaMailSender.createMimeMessage();
            MimeMessageHelper message = new MimeMessageHelper(mimeMessage, StandardCharsets.UTF_8.name());
            message.setText(parameter);
            message.setSubject(mailProperties.getSubject());
            message.setTo(mailProperties.getTo());
            message.setCc(mailProperties.getCc());
            message.setFrom(mailProperties.getFrom());
            javaMailSender.send(mimeMessage);
        }
        catch (MessagingException ex) {
            logger.error("Error sending mail notification", ex);
            return new DefaultNotifierResult(ex);
        }
        return new DefaultNotifierResult(true);
    }
}
