package com.hncy.pageable.spring.boot.starter.core;

import com.hncy.pageable.spring.boot.starter.PageProperties;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @Author: ying.dong
 * @Date: 2021/5/6 14:58
 */
@Slf4j
public class PageableInterceptor implements HandlerInterceptor {

    private final PageProperties properties;

    public PageableInterceptor(PageProperties properties) {
        this.properties = properties;
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        String pageNum = request.getParameter(properties.getPageNumName());
        String pageSize = request.getParameter(properties.getPageSizeName());
        if (checkPageParam(pageNum) && checkPageParam(pageSize)) {
            int size = Integer.parseInt(pageSize);
            if (!properties.isAllowPageSizeOverFlow() && properties.getMaxPageSize() < size) {
                throw new IllegalArgumentException("page size exceeded. expected max [" + properties.getMaxPageSize() + "], but [" + size + "].");
            }
            log.debug("find page params in request paramters, pageNum: {}, pageSize: {}", pageNum, pageSize);
            PageContext.setPageInfo(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        }
        return true;
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {
        PageContext.clear();
    }

    /**
     *
     * @param param
     * @return
     */
    private boolean checkPageParam(String param) {
        if (StringUtils.isBlank(param)) {
            return false;
        }
        return StringUtils.isNumeric(param);
    }
}
