package com.hncy.sentinel.spring.boot.starter;

import com.alibaba.cloud.sentinel.gateway.ConfigConstants;
import com.alibaba.csp.sentinel.adapter.gateway.zuul.fallback.ZuulBlockFallbackProvider;
import com.alibaba.csp.sentinel.adapter.spring.webmvc.callback.BlockExceptionHandler;
import com.alibaba.csp.sentinel.util.StringUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hncy.sentinel.spring.boot.starter.message.DefaultBlockMessage;
import com.hncy.sentinel.spring.boot.starter.message.ResponseMessage;
import com.netflix.zuul.http.ZuulServlet;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.web.servlet.DispatcherServlet;

import java.io.PrintWriter;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * @Author: ying.dong
 * @Date: 2021/6/30 16:09
 */
@Slf4j
@Configuration
public class SentinelAutoConfiguration {

    @Configuration
    static class BlockMessageConfiguration {

        @ConditionalOnMissingBean(ResponseMessage.class)
        @Bean
        public ResponseMessage blockMessage() {
            return new DefaultBlockMessage();
        }
    }

    @Configuration
    @ConditionalOnClass(DispatcherServlet.class)
    static class WebMvcHandler {

        @ConditionalOnMissingBean(BlockExceptionHandler.class)
        @Bean
        public BlockExceptionHandler webmvcBlockExceptionHandler(ObjectMapper objectMapper, ResponseMessage blockMessage) {
            return (request, response, e) -> {
                StringBuffer url = request.getRequestURL();

                if ("GET".equals(request.getMethod()) && StringUtil.isNotBlank(request.getQueryString())) {
                    url.append("?").append(request.getQueryString());
                }

                log.warn("too many requests. url: {}", url);

                int status = HttpStatus.TOO_MANY_REQUESTS.value();
                response.setStatus(status);
                response.setContentType(MediaType.APPLICATION_JSON_UTF8_VALUE);
                Map<String, Object> blockedMessage = new LinkedHashMap<>();
                blockedMessage.put("code", blockMessage.getCode());
                blockedMessage.put("message", blockMessage.getMessage());
                String jsonString = objectMapper.writeValueAsString(blockedMessage);
                PrintWriter out = response.getWriter();
                out.print(jsonString);
                out.flush();
                out.close();
            };
        }
    }

    @Configuration
    @ConditionalOnClass(ZuulServlet.class)
    @ConditionalOnProperty(prefix = ConfigConstants.ZUUL_PREFIX, name = "enabled",
            havingValue = "true", matchIfMissing = true)
    static class ZuulBlockHandler {

        @ConditionalOnMissingBean(ZuulBlockFallbackProvider.class)
        @Bean
        ZuulBlockFallbackProvider zuulBlockFallbackProvider(ResponseMessage blockMessage) {
            return new CommonZuulBlockFallbackProvider(blockMessage);
        }
    }
}
