package com.hncy.sms.spring.boot.starter.aliyun;

import com.hncy.sms.spring.boot.starter.aliyun.client.AliyunSmsClient;
import com.hncy.sms.spring.boot.starter.aliyun.request.AliyunSmsSendRequest;
import com.hncy.sms.spring.boot.starter.aliyun.response.AliyunSmsSendResponse;
import com.hncy.sms.spring.boot.starter.aliyun.util.AliyunSignatureBuilder;
import com.hncy.sms.spring.boot.starter.aliyun.util.AliyunSmsProperties;
import com.hncy.sms.spring.boot.starter.core.SmsManager;
import com.hncy.sms.spring.boot.starter.core.exception.SmsException;
import com.hncy.sms.spring.boot.starter.core.request.BatchSmsRequest;
import com.hncy.sms.spring.boot.starter.core.request.SingleSmsRequest;
import com.hncy.sms.spring.boot.starter.core.response.SmsResponse;
import org.springframework.http.HttpMethod;

/**
 * 阿里云短信
 * @Author: ying.dong
 * @Date: 2021/5/11 14:37
 */
public class AliyunSmsManager implements SmsManager {

    private final AliyunSmsProperties properties;

    private final AliyunSmsClient aliyunSmsClient;

    private final AliyunSignatureBuilder signatureBuilder;

    public AliyunSmsManager(AliyunSmsProperties properties, AliyunSmsClient aliyunSmsClient) {
        this.properties = properties;
        this.aliyunSmsClient = aliyunSmsClient;
        this.signatureBuilder = new AliyunSignatureBuilder(properties.getAccessKey(), properties.getAccessSecret());
    }

    @Override
    public SmsResponse send(SingleSmsRequest request) throws SmsException {
        AliyunSmsSendRequest signedRequest = prepareSign((AliyunSmsSendRequest) request);
        AliyunSmsSendResponse aliyunSmsSendResponse;
        try {
            aliyunSmsSendResponse = aliyunSmsClient.sendSms(signedRequest);
        } catch (Exception e) {
            throw new SmsException(e.getMessage(), e);
        }
        if (!aliyunSmsSendResponse.isSuccess()) {
            throw new SmsException(aliyunSmsSendResponse.getMessage());
        }
        return aliyunSmsSendResponse;
    }

    @Override
    public SmsResponse batchSend(BatchSmsRequest request) throws SmsException {
        return send((AliyunSmsSendRequest)request);
    }


    private AliyunSmsSendRequest prepareSign(AliyunSmsSendRequest request) throws SmsException {
        request.setAccessKeyId(properties.getAccessKey());
        String signature;
        try {
            signature = signatureBuilder.buildSignature(request, HttpMethod.POST);
        } catch (Exception e) {
            throw new SmsException("sign error.", e);
        }
        request.setSignature(signature);
        return request;
    }

}
