package com.hncy.sms.spring.boot.starter.aliyun.request;

import cn.hutool.core.util.StrUtil;
import com.hncy.jackson.spring.boot.starter.core.JSONObject;
import com.hncy.sms.spring.boot.starter.core.request.BatchSmsRequest;
import com.hncy.sms.spring.boot.starter.core.request.SingleSmsRequest;
import lombok.Data;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;

import java.util.Collection;
import java.util.Map;

/**
 * @Author: ying.dong
 * @Date: 2021/5/11 14:41
 */
@Data
@ToString(callSuper = true)
@Slf4j
public class AliyunSmsSendRequest extends AliyunSmsBaseRequest implements SingleSmsRequest, BatchSmsRequest {

    /**
     * 接收短信的手机号码。
     *
     * 格式：
     *
     * 国内短信：+/+86/0086/86或无任何前缀的11位手机号码，例如1381111****。
     * 国际/港澳台消息：国际区号+号码，例如852000012****。
     * 上限为1000个手机号码。批量调用相对于单条调用及时性稍有延迟。
     * 必填
     */
    private Collection<String> PhoneNumbers;

    /**
     * 短信签名名称。请在控制台国内消息或国际/港澳台消息页面中的签名管理页签下签名名称一列查看。
     * 必填
     */
    private String SignName;

    /**
     * 短信模板ID。请在控制台国内消息或国际/港澳台消息页面中的模板管理页签下模板CODE一列查看。
     * 必填
     */
    private String TemplateCode;

    /**
     * 短信模板变量对应的实际值
     * 非必填
     */
    private Map<String, Object> TemplateParam;

    /**
     * 上行短信扩展码，上行短信，指发送给通信服务提供商的短信，用于定制某种服务、完成查询，或是办理某种业务等，需要收费的，按运营商普通短信资费进行扣费。
     * 非必填
     */
    private String SmsUpExtendCode;

    /**
     * 外部流水扩展字段
     * 非必填
     */
    private String OutId;

    public String getPhoneNumbers() {
        return StrUtil.join(",", PhoneNumbers);
    }

    public String getTemplateParam() {
        return JSONObject.getInstance().toJSONString(TemplateParam);
    }
}
