package com.hncy.common.dto;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.hncy.common.exception.common.ErrorCode;
import com.hncy.common.exception.common.GlobalErrorConstants;
import com.hncy.common.exception.custom.BaseBusinessException;
import com.hncy.common.exception.custom.BusinessException;
import com.hncy.common.exception.custom.LocalBusinessException;
import lombok.Data;

import java.io.Serializable;

/**
 * note: 此取名用来兼容已有的代码
 * @Author: ying.dong
 * @Date: 2021/4/19 15:10
 */
@Data
public class ResultBean<T> implements Serializable {
    private static final long serialVersionUID = 2251319656486478070L;

    /**
     * 错误码
     *
     * @see ErrorCode#getCode()
     */
    private Integer code;
    /**
     * 返回数据
     */
    private T data;
    /**
     * 错误提示，用户可阅读
     *
     * @see ErrorCode#getMessage()
     */
    private String message;

    public static <T> ResultBean<T> fail() {
        throw new BusinessException();
    }

    public static <T> ResultBean<T> fail(int code) {
        throw new LocalBusinessException(code);
    }

    public static <T> ResultBean<T> fail(String message) {
        throw new BusinessException(message);
    }

    public static <T> ResultBean<T> fail(Integer code, String message) {
        throw new BusinessException(message);
    }

    public static <T> ResultBean<T> fail(ErrorCode errorCode) {
        throw new BusinessException(errorCode);
    }

    public static <T> ResultBean<T> fail(Throwable throwable) {
        if (throwable instanceof BaseBusinessException) {
            throw (BaseBusinessException)throwable;
        }
        throw new BusinessException(throwable);
    }

    public static <T> ResultBean<T> success() {
        ResultBean<T> result = new ResultBean<>();
        result.code = GlobalErrorConstants.SUCCESS.getCode();
        result.message = "操作成功";
        return result;
    }

    public static <T> ResultBean<T> success(T data) {
        ResultBean<T> result = new ResultBean<>();
        result.code = GlobalErrorConstants.SUCCESS.getCode();
        result.data = data;
        result.message = "操作成功";
        return result;
    }

    public Integer getCode() {
        return code;
    }

    public void setCode(Integer code) {
        this.code = code;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public T getData() {
        return data;
    }

    public void setData(T data) {
        this.data = data;
    }

    @JsonIgnore   // 避免序列化
    public boolean isSuccess() {
        return GlobalErrorConstants.SUCCESS.getCode().equals(code);
    }

    @JsonIgnore // 避免序列化
    public boolean isError() {
        return !isSuccess();
    }

}
