package com.hncy.common.jwt.verifier;

import cn.hutool.crypto.KeyUtil;
import cn.hutool.jwt.JWT;
import cn.hutool.jwt.signers.AlgorithmUtil;
import cn.hutool.jwt.signers.JWTSigner;
import cn.hutool.jwt.signers.JWTSignerUtil;

import java.security.PublicKey;
import java.security.spec.KeySpec;

/**
 * RSA算法
 * @Author: ying.dong
 * @Date: 2021/10/13 09:37
 */
public abstract class RSAVerifier extends BaseJWTVerifier {

    @Override
    protected void prepareJWT(JWT jwt) {
        String algorithm = AlgorithmUtil.getAlgorithm(jwt.getAlgorithm());
        KeySpec pubKeySpec = getPublicKeySpec(jwt);
        PublicKey publicKey = KeyUtil.generatePublicKey(algorithm, pubKeySpec);
        JWTSigner signer = JWTSignerUtil.createSigner(algorithm, publicKey);
        jwt.setSigner(signer);
    }

    protected abstract KeySpec getPublicKeySpec(JWT jwt);
}
