package com.hncy.deploy.command;

import cn.hutool.core.util.StrUtil;
import com.hncy.deploy.dto.ApplicationInfo;
import com.hncy.deploy.dto.ApplicationProcess;
import com.hncy.deploy.service.DeployService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.shell.standard.ShellComponent;
import org.springframework.shell.standard.ShellMethod;
import org.springframework.shell.standard.ShellOption;

import java.util.List;

/**
 * @Author: ying.dong
 * @Date: 2021/5/15 15:04
 */
@Slf4j
@ShellComponent
public class DeployCommand {

    @Autowired
    private DeployService deployService;

    @ShellMethod(value = "list services", key = "ls")
    public List<ApplicationInfo> listServices() {
        return deployService.list();
    }

    @ShellMethod(value = "java process list", key = "ps")
    public List<ApplicationProcess> ps(@ShellOption(defaultValue = ShellOption.NULL) String name) {
        if (StrUtil.isNotBlank(name)) {
            name = StrUtil.trim(name);
            return deployService.ps(name);
        }
        return deployService.ps();
    }

    @ShellMethod(value = "shutdown java application", key = "shutdown")
    public void shutdown(@ShellOption(defaultValue = ShellOption.NULL) String name,
                         @ShellOption(defaultValue = ShellOption.NULL) Integer id) {
        if (StrUtil.isNotBlank(name)) {
            name = StrUtil.trim(name);
            deployService.shutdownGraceful(name);
        }
        if (id != null) {
            deployService.shutdownGraceful(id);
        }
    }

    @ShellMethod(value = "start java process", key = "start")
    public void start(String name) {
        name = StrUtil.trim(name);
        deployService.start(name);
    }

    @ShellMethod(value = "restart java process", key = "restart")
    public void restart(String name) {
        name = StrUtil.trim(name);
        deployService.restart(name);
    }

    @ShellMethod(value = "health check", key = "health")
    public String health(String name) {
        name = StrUtil.trim(name);
        return deployService.health(name);
    }

    @ShellMethod(value = "upgrade application", key = "upgrade")
    public void upgrade(String name) {
        name = StrUtil.trim(name);
        deployService.upgrade(name);
    }

    @ShellMethod(value = "rollback application", key = "rollback")
    public void rollback(String name) {
        name = StrUtil.trim(name);
        deployService.rollback(name);
    }
}
