package com.hncy.deploy.service;

import com.hncy.deploy.dto.ApplicationInfo;
import com.hncy.deploy.dto.ApplicationProcess;

import java.util.List;

/**
 * @Author: ying.dong
 * @Date: 2021/5/18 10:10
 */
public interface DeployService {

    /**
     * 获取配置的服务信息
     * @return
     */
    List<ApplicationInfo> list();

    /**
     * 根据服务名获取进程
     * @param name
     * @return
     */
    List<ApplicationProcess> ps(String name);

    /**
     * 根据进程ID获取进程
     * @param id
     * @return
     */
    ApplicationProcess ps(Integer id);

    /**
     * 获取所有JAVA应用进程
     * @return
     */
    List<ApplicationProcess> ps();

    /**
     * 根据服务名查看进程是否存在
     * @param name
     * @return
     */
    boolean isProcessExists(String name);

    /**
     * 根据服务名关闭
     * @param name
     * @@param timeoutInSeconds
     */
    boolean shutdownGraceful(String name);

    /**
     * 根据进程ID关闭
     * @param id
     */
    boolean shutdownGraceful(Integer id);

    /**
     * 根据ID强制关闭
     * @param id
     * @return
     */
    boolean shutdownForce(Integer id);

    /**
     * 根据服务名启动
     * @param name
     */
    void start(String name);

    /**
     * 根据服务名重启
     * @param name
     */
    void restart(String name);

    /**
     * 健康检查
     * @param name
     * @return
     */
    String health(String name);

    /**
     * 健康检查
     * @return
     */
    boolean isHealthy(String name);

    /**
     * 升级应用
     * @param name
     */
    void upgrade(String name);

    /**
     * 回滚应用
     * @param name
     */
    void rollback(String name);
}
