package com.hncy.service.wechat.applet.api.feign;

import com.hncy.common.constant.GlobalConstant;
import com.hncy.common.dto.ResultBean;
import com.hncy.service.wechat.applet.api.constant.ServiceWxAppletApiConstant;
import com.hncy.service.wechat.applet.api.dto.output.Code2SessionOutput;
import com.hncy.service.wechat.applet.api.vo.WxUserInfo;
import com.hncy.service.wechat.applet.api.vo.WxUserVo;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * @Author: cs
 * @Date: 2021/10/28 14:33
 */
@FeignClient(name = ServiceWxAppletApiConstant.FEIGN_CLIENT_NAME,
        path = GlobalConstant.NameSpace.INTERNAL,
        qualifier = ServiceWxAppletApiConstant.QUALIFIER)
public interface WxAppletFeignClient {

    @ApiOperation(value = "根据微信用户id查询openid(内部调用)")
    @PostMapping(value = "/wx-user/getOpenId")
    ResultBean<WxUserInfo> getOpenId(@RequestParam("wxUserId") String wxUserId);

    @ApiOperation(value = "根据openid查询用户是否存在(内部调用)")
    @PostMapping(value = "/wx-user/booleanWxUserByOpenId")
    ResultBean<WxUserVo> booleanWxUserByOpenId(@ApiParam(name = "openId", value = "openId", required = true)@RequestParam("openId") String openId);

    @ApiOperation(value = "根据openid获取小程序用户信息(内部调用)")
    @PostMapping(value = "/wx-user/getWxUserInfo", produces = "application/json;charset=UTF-8")
    ResultBean<WxUserVo> getWxUserInfo(@ApiParam(name = "openId", value = "openId", required = true)@RequestParam("openId") String openId);

    @ApiOperation(value = "保存微信用户(内部调用)")
    @PostMapping(value = "/wx-user/saveWxUser")
    ResultBean<WxUserVo> saveWxUser(@ApiParam(name = "用户唯一标识", value = "openId", required = true)@RequestParam("openId") String openId,
                                    @ApiParam(name = "用户在开放平台的唯一标识符", value = "unionId", required = false)@RequestParam(value = "unionId", required = false) String unionId);

    @ApiOperation(value = "根据微信用户id获取用户名字(内部调用)")
    @GetMapping(value = "/wx-user/getNickNameById")
    ResultBean<String> getNickNameById(@ApiParam(name = "id", value = "id", required = true)@RequestParam("id") String id);

}
