package com.hncy.service.wechat.applet.biz.controller;

import cn.hutool.core.io.FileTypeUtil;
import com.hncy.service.wechat.applet.biz.dto.ComponentPictureDto;
import com.hncy.service.wechat.applet.biz.mapper.ComponentPictureMapper;
import com.hncy.service.wechat.applet.biz.utils.FastdfsClientUtil;
import com.hncy.service.wechat.applet.biz.vo.PatrolInspectionComponentsSumVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import com.hncy.common.dto.ResultBean;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.hncy.service.wechat.applet.biz.entity.ComponentPicture;
import com.hncy.service.wechat.applet.biz.service.ComponentPictureService;
import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import java.util.List;

import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

/**
* <p>
    * 部件图片表controller
    * </p>
*
* @author cs
* @since 2021-11-04
*/
@RestController
@RequestMapping("/component-picture")
@Slf4j
@Api(tags = "部件图片表")
public class ComponentPictureController {

    //文件储存ip地址
    @Value("${file.upload}")
    private String basePath;

    //缩略图宽度
    @Value("${fdfs.thumb-image.width}")
    private String thumbWidth;

    //缩略图高度
    @Value("${fdfs.thumb-image.height}")
    private String thumbHeight;

    @Autowired
    private ComponentPictureService componentPictureService;
    @Autowired
    private ComponentPictureMapper componentPictureMapper;

    @Autowired
    private FastdfsClientUtil fastdfsClientUtil;

    @ApiOperation(value = "上传部件图片")
    @PostMapping(value = "/uploadComponentPicture")
    ResultBean<ComponentPicture> uploadComponentPicture(@ApiParam(name = "file",value = "部件图片")@RequestParam(value = "file")MultipartFile file,
                                      @ApiParam(name = "storageForm",value = "存储形式：1.部件采集 2.问题上报")@RequestParam(value = "storageForm")String storageForm) throws Exception {
        //上传文件 返回文件地址
        String upload=null;
        //文件名
        String fileName = file.getOriginalFilename();
        //后缀类型并转为小写
        String suffixName = fileName.substring(fileName.lastIndexOf(".")).substring(1).toLowerCase();
        //图片上传
        upload = fastdfsClientUtil.upload(file);
        //文件大小
        long size = file.getSize();
        //文件全地址
        String allPath = basePath +upload;
        ComponentPicture componentPicture = new ComponentPicture();
        componentPicture.setComponentPictureSize(size+"");
        componentPicture.setComponentPictureUrl(allPath);
        componentPicture.setComponentPictureName(fileName);
        componentPicture.setComponentPictureType(suffixName);
        componentPictureMapper.insert(componentPicture);
        return ResultBean.success(componentPicture);
    }

    @ApiOperation(value = "删除部件图片(包括批量删除)")
    @DeleteMapping(value = "/deletePictures")
    public ResultBean deletePictures(@ApiParam(name = "ids", value = "图片id集合", required = true)@RequestParam("ids") List<String> ids){
        return componentPictureService.deletePictures(ids);
    }


}
