package com.hncy.service.wechat.applet.biz.mini.config;

import com.hncy.common.constant.GlobalConstant;
import com.hncy.lock.spring.boot.starter.core.Lock;
import com.hncy.lock.spring.boot.starter.distribute.ReentrantRedisLock;
import com.hncy.redis.spring.boot.starter.core.RedisRepository;
import com.hncy.service.wechat.applet.biz.mini.common.context.DefaultMiniProgramContext;
import com.hncy.service.wechat.applet.biz.mini.common.context.MiniProgramContext;
import com.hncy.service.wechat.applet.biz.mini.common.store.AccessTokenStore;
import com.hncy.service.wechat.applet.biz.mini.common.store.RedisAccessTokenStore;
import com.hncy.service.wechat.applet.biz.mini.feign.MiniProgramOriginalFeignClient;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @Author: ying.dong
 * @Date: 2021/6/15 16:01
 */
@Configuration
@EnableConfigurationProperties(MiniProgramProperties.class)
public class MiniProgramConfiguration {

    @Bean
    AccessTokenStore accessTokenStore(RedisRepository<String> redisRepository, MiniProgramProperties miniProgramProperties) {
        String key = GlobalConstant.RedisNameSpace.SERVICE_APPLET + "MINI_PROGRAM_ACCESS_TOKEN_" + miniProgramProperties.getAppId();
        return new RedisAccessTokenStore(key, miniProgramProperties.getExpireInSeconds(), redisRepository);
    }

    @Bean
    MiniProgramContext miniProgramContext(MiniProgramProperties programProperties, AccessTokenStore accessTokenStore,
                                          MiniProgramOriginalFeignClient miniProgramOriginalFeignClient, Lock lock) {
        return new DefaultMiniProgramContext(programProperties, accessTokenStore, miniProgramOriginalFeignClient, lock);
    }

    @Bean
    Lock lock(RedisRepository<String> redisRepository) {
        return new ReentrantRedisLock(redisRepository);
    }

}
