package com.hncy.service.wechat.applet.biz.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.StrUtil;
import com.hncy.common.dto.ResultBean;
import com.hncy.common.exception.custom.BusinessException;
import com.hncy.service.wechat.applet.biz.entity.*;
import com.hncy.service.wechat.applet.biz.mapper.*;
import com.hncy.service.wechat.applet.biz.service.ComponentIssuesReportService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hncy.service.wechat.applet.biz.service.ComponentsRemoteService;
import com.hncy.service.wechat.applet.biz.utils.FastdfsClientUtil;
import com.hncy.service.wechat.applet.biz.vo.RegTypesVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * <p>
 * 部件问题上报 服务实现类
 * </p>
 *
 * @author cs
 * @since 2021-11-02
 */
@Service
@Slf4j
public class ComponentIssuesReportServiceImpl extends ServiceImpl<ComponentIssuesReportMapper, ComponentIssuesReport> implements ComponentIssuesReportService {

    @Autowired
    private ComponentsRemoteService componentsRemoteService;
    @Autowired
    private ComponentIssuesReportMapper componentIssuesReportMapper;
    @Autowired
    private ComponentPictureMapper componentPictureMapper;
    @Autowired
    private ComponentPictureRelationMapper componentPictureRelationMapper;
    @Autowired
    private PlatformAuditMapper platformAuditMapper;
    @Autowired
    private ComponentDetailMapper componentDetailMapper;
    @Autowired
    private ComponentIssueReportRelationMapper componentIssueReportRelationMapper;


    @Override
    public ResultBean<List<RegTypesVo>> selectAllRegTypes(String componentReportCode, String componentReportName) {
        if(StrUtil.isBlank(componentReportCode) || StrUtil.isBlank(componentReportName)) {
            throw new BusinessException("请选择部件类型");
        }
        List<RegTypesVo> regTypesVos  = componentIssuesReportMapper.selectAllRegTypes(componentReportCode,componentReportName);
        log.info(regTypesVos.toString());
        return ResultBean.success(regTypesVos);
    }

    @Override
    public ResultBean saveComponentIssues(String regCondCode, String regCondName, String issueDescription, String componentReportCode, String componentReportName) {
        ComponentIssuesReport componentIssuesReport = new ComponentIssuesReport();
        componentIssuesReport.setRegCondCode(regCondCode);
        componentIssuesReport.setRegCondName(regCondName);
        componentIssuesReport.setIssueDescription(issueDescription);
        componentIssuesReport.setComponentReportCode(componentReportCode);
        componentIssuesReport.setComponentReportName(componentReportName);
        componentIssuesReport.setGmtCreate(DateTime.now());
        componentIssuesReport.setDeleted(0);
        componentIssuesReportMapper.insert(componentIssuesReport);
        return ResultBean.success();
    }

    @Override
    public ResultBean componentReport(String componentName, String componentCode, String componentSuperName, String componentSuperCode, String componentTypeName, String componentTypeCode, String componentAddress, String xCode, String yCode, LocalDateTime reportTime, String regCondCode, String regCondName, String issueDescription, List<String> pictureIds, String reportNickName, String componentCollectionWays) {
        //部件详情
        ComponentDetail componentDetail = new ComponentDetail();
        componentDetail.setComponentName(componentName);
        componentDetail.setComponentCode(componentCode);
        componentDetail.setComponentSuperName(componentSuperName);
        componentDetail.setComponentSuperCode(componentSuperCode);
        componentDetail.setComponentTypeName(componentTypeName);
        componentDetail.setComponentTypeCode(componentTypeCode);
        componentDetail.setComponentAddress(componentAddress);
        componentDetail.setComponentCollectionWays(componentCollectionWays);
        componentDetail.setXCode(xCode);
        componentDetail.setYCode(yCode);
        componentDetail.setPatrolInspectionTime(null);
        componentDetail.setDeleted(0);
        componentDetail.setGmtCreate(null);
        componentDetail.setCreateBy(reportNickName);
        int insert = componentDetailMapper.insert(componentDetail);
        if(insert == 0) {
            throw new BusinessException("部件问题上报失败，请重新上报");
        }
        String componentId = componentDetail.getId();

        //部件问题上报
        ComponentIssuesReport componentIssuesReport = new ComponentIssuesReport();
        componentIssuesReport.setComponentReportName(componentName);
        componentIssuesReport.setComponentReportCode(componentCode);
        componentIssuesReport.setRegCondName(regCondName);
        componentIssuesReport.setRegCondCode(regCondCode);
        componentIssuesReport.setIssueDescription(issueDescription);
        componentIssuesReport.setDeleted(0);
        componentIssuesReport.setGmtCreate(DateTime.now());
        int insert1 = componentIssuesReportMapper.insert(componentIssuesReport);
        if(insert1 == 0) {
            throw new BusinessException("部件问题上报失败，请重新上报");
        }
        String issueId = componentIssuesReport.getId();

        //部件采集与图片关联
        pictureIds.stream().forEach(pictureId ->{
            ComponentPictureRelation componentPictureRelation = new ComponentPictureRelation();
            componentPictureRelation.setComponentDetailId(null);
            componentPictureRelation.setComponentPictureId(pictureId);
            componentPictureRelation.setComponentIssuesReportId(issueId);
            componentPictureRelation.setDeleted(0);
            componentPictureRelation.setGmtCreate(DateTime.now());
            int insert2 = componentPictureRelationMapper.insert(componentPictureRelation);
            if(insert2 == 0) {
                throw new BusinessException("部件问题上报失败，请重新上报");
            }
        });

        //新增关联关系
        ComponentIssueReportRelation componentIssueReportRelation = new ComponentIssueReportRelation();
        componentIssueReportRelation.setComponentIssuesId(issueId);
        componentIssueReportRelation.setComponentObjId(componentId);
        componentIssueReportRelation.setDeleted(0);
        componentIssueReportRelation.setGmtCreate(DateTime.now());
        int insert2 = componentIssueReportRelationMapper.insert(componentIssueReportRelation);
        if(insert2 == 0) {
            throw new BusinessException("部件问题上报失败，请重新上报");
        }

        //如果部件案件条件不是完好则上报给城市管理系统
        if(!"完好".equalsIgnoreCase(regCondName) && CollectionUtil.isNotEmpty(pictureIds)) {
            Map<String, Object> map = new HashMap<>();
            //获取图片
            Map<String, String> fileMap = new HashMap<>();
            pictureIds.stream().forEach(pictureId -> {
                ComponentPicture componentPicture = componentPictureMapper.selectById(pictureId);
                String imageToBase64 = FastdfsClientUtil.image2Base64(componentPicture.getComponentPictureUrl());
                fileMap.put(componentPicture.getComponentPictureName(),imageToBase64);
            });
            //获取案件主键：32位UUID
            String caseInfoId = UUID.randomUUID().toString().replaceAll("-", "");
            map.put("fileMap",fileMap);
            map.put("caseInfoId",caseInfoId);
            map.put("addr",componentAddress);
            map.put("problemDesc",issueDescription);
            map.put("bigClassCode",componentSuperCode);
            map.put("bigClassName",componentSuperName);
            map.put("subClassCode",componentCode);
            map.put("subClassName",componentName);
            map.put("regCondCode",regCondCode);
            map.put("regCondName",regCondName);
            map.put("lon",xCode);
            map.put("lat",yCode);
            componentsRemoteService.reportByAR(map);
        }

        return ResultBean.success("部件问题上报成功");

    }
}
