package com.hncy.service.wechat.applet.biz.service.impl;

import cn.hutool.core.date.DateTime;
import com.hncy.common.dto.ResultBean;
import com.hncy.common.exception.custom.BusinessException;
import com.hncy.service.wechat.applet.biz.dto.ComponentPictureDto;
import com.hncy.service.wechat.applet.biz.entity.ComponentPicture;
import com.hncy.service.wechat.applet.biz.mapper.ComponentPictureMapper;
import com.hncy.service.wechat.applet.biz.service.ComponentPictureService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hncy.service.wechat.applet.biz.utils.FastdfsClientUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 部件图片表 服务实现类
 * </p>
 *
 * @author cs
 * @since 2021-11-04
 */
@Service
public class ComponentPictureServiceImpl extends ServiceImpl<ComponentPictureMapper, ComponentPicture> implements ComponentPictureService {

    //文件储存ip地址
    @Value("${file.upload}")
    private String basePath;

    @Autowired
    private ComponentPictureMapper componentPictureMapper;

    @Autowired
    private FastdfsClientUtil fastdfsClientUtil;

    @Override
    public ResultBean<List<ComponentPicture>> uploadComponentPicture(List<ComponentPictureDto> componentPictureDtos) {
        List<ComponentPicture> componentPictureList = new ArrayList<>();
        componentPictureDtos.stream().forEach(componentPictureDto -> {
            //上传文件 返回文件地址
            String uploadTag=null;
            try {
                uploadTag = fastdfsClientUtil.upload(componentPictureDto.getComponentPicture());
            } catch (Exception e) {
                log.error("图片上传失败" + e.getMessage());
                throw new BusinessException("图片上传失败");
            }
            String pictureUrl = basePath + uploadTag;
            ComponentPicture componentPicture = new ComponentPicture();
            componentPicture.setComponentPictureName(componentPictureDto.getComponentPictureName());
            componentPicture.setComponentPictureSize(componentPictureDto.getComponentPictureSize());
            componentPicture.setComponentPictureType(componentPictureDto.getComponentPictureType());
            componentPicture.setComponentPictureUrl(pictureUrl);
            componentPicture.setStorageForm(componentPictureDto.getStorageForm());
            componentPicture.setDeleted(0);
            componentPicture.setGmtCreate(DateTime.now());
            componentPictureMapper.insert(componentPicture);
            componentPictureList.add(componentPicture);
        });
        return ResultBean.success(componentPictureList);
    }

    @Override
    public ResultBean deletePictures(List<String> ids) {
        List<ComponentPicture> componentPictureList = componentPictureMapper.selectBatchIds(ids);
        componentPictureList
                .stream()
                .filter(componentPicture -> (componentPicture.getDeleted() == 0))
                .forEach(componentPicture -> {
                    int i = componentPictureMapper.deletePictures(componentPicture.getId());
                    if(i == 0) {
                        throw new BusinessException("文件删除失败");
                    }else {
                        fastdfsClientUtil.deleteFile(componentPicture.getComponentPictureUrl());
                    }
                });
        return ResultBean.success("文件删除成功");


    }
}
