package com.hncy.service.wechat.applet.biz.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.hncy.common.constant.GlobalConstant;
import com.hncy.common.dto.ResultBean;
import com.hncy.common.exception.custom.BusinessException;
import com.hncy.common.exception.custom.UnAuthorizedException;
import com.hncy.redis.spring.boot.starter.core.RedisRepository;
import com.hncy.service.wechat.applet.api.dto.output.Code2SessionOutput;
import com.hncy.service.wechat.applet.api.feign.WxAppletFeignClient;
import com.hncy.service.wechat.applet.api.vo.WxUserInfo;
import com.hncy.service.wechat.applet.api.vo.WxUserVo;
import com.hncy.service.wechat.applet.biz.entity.WxUser;
import com.hncy.service.wechat.applet.biz.mapper.WxUserMapper;
import com.hncy.service.wechat.applet.biz.mini.config.MiniProgramProperties;
import com.hncy.service.wechat.applet.biz.mini.service.MiniProgramService;
import com.hncy.service.wechat.applet.biz.service.ComponentsRemoteService;
import com.hncy.service.wechat.applet.biz.service.WxUserService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hncy.service.wechat.applet.biz.vo.JudegAccesstokenVo;
import com.hncy.service.wechat.applet.biz.vo.WechatAccesstokenVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.sql.Struct;
import java.time.LocalDateTime;

/**
 * <p>
 * 微信小程序用户表 服务实现类
 * </p>
 *
 * @author cs
 * @since 2021-10-27
 */
@Service
@Slf4j
public class WxUserServiceImpl extends ServiceImpl<WxUserMapper, WxUser> implements WxUserService {

    @Autowired
    private WxUserMapper wxUserMapper;
    @Autowired
    private WxAppletFeignClient wxAppletFeignClient;
    @Autowired
    private MiniProgramService miniProgramService;
    @Autowired
    private RedisRepository<String> redisRepository;
    @Autowired
    private MiniProgramProperties miniProgramProperties;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultBean saveWxUserInfo(String openId, String avatar, String nickname, String sexCode, String sexName, String tel) {

        //判断该手机号是否被其他用户注册了
        WxUser wxUserTel = wxUserMapper.selectByTel(tel);
        if (ObjectUtil.isNotNull(wxUserTel)) {
            throw new BusinessException("该手机号已经被其他用户注册,请重新更换手机号");
        }
        // 通过openId获取用户信息;
        WxUser wxUser = wxUserMapper.getWxUserInfoByopenId(openId);
        if(ObjectUtil.isNull(wxUser)) {
            log.info("用户不存在,注册");
            WxUser newWxUser = new WxUser();
            newWxUser.setOpenId(openId);
            newWxUser.setDeleted(0);
            newWxUser.setAvatar(avatar);
            newWxUser.setGmtCreate(DateTime.now());
            newWxUser.setLoginDate(LocalDateTime.now());
            newWxUser.setSexCode(sexCode);
            newWxUser.setSexName(sexName);
            newWxUser.setNickname(nickname);
            newWxUser.setTel(tel);
            newWxUser.setCreateBy(openId);
            newWxUser.setRole(null);
            //录入数据库
            wxUserMapper.insert(newWxUser);
            return ResultBean.success("用户注册成功");
        }else {
            log.info("用户已存在，更新信息");
            WxUser newWxUser = new WxUser();
            newWxUser.setId(wxUser.getId());
            newWxUser.setOpenId(wxUser.getOpenId());
            newWxUser.setDeleted(0);
            newWxUser.setAvatar(avatar);
            newWxUser.setLoginDate(LocalDateTime.now());
            newWxUser.setSexCode(sexCode);
            newWxUser.setSexName(sexName);
            newWxUser.setNickname(nickname);
            newWxUser.setTel(tel);
            newWxUser.setUpdateBy(openId);
            newWxUser.setGmtModified(DateTime.now());
            newWxUser.setRole(null);
            wxUserMapper.updateById(newWxUser);
            return ResultBean.success("用户更新成功");
        }
    }

    @Override
    public ResultBean<WxUserVo> booleanWxUserByOpenId(String openId) {
        WxUserVo wxUserVo = new WxUserVo();
        WxUser wxUser = wxUserMapper.selectOpenId(openId);
        if(null == wxUser) {
            return ResultBean.success(null);
        }
        BeanUtils.copyProperties(wxUser, wxUserVo);
        return ResultBean.success(wxUserVo);
    }

    @Override
    public ResultBean<WxUserInfo> getOpenId(String wxUserId) {
        WxUserInfo wxUserInfo = new WxUserInfo();
        String openId = wxUserMapper.getOpenId(wxUserId);
        if(!StringUtils.isEmpty(openId)) {
            wxUserInfo.setOpenId(openId);
        }
        return ResultBean.success(wxUserInfo);
    }

    @Override
    public ResultBean<String> getNickNameById(String id) {
        WxUser wxUser = wxUserMapper.selectById(id);
        return ResultBean.success(wxUser.getNickname());
    }

    @Override
    public ResultBean<WechatAccesstokenVo> appletLogin(String code) {
        log.info(code);
        Code2SessionOutput data = miniProgramService.login(code);

        if(null != data.getErrcode()  && StrUtil.isNotBlank(data.getErrmsg())) {
            throw new BusinessException("小程序登录异常，请重新获取授权");
        }
        if (StrUtil.isBlank(data.getOpenid())) {
            throw new BusinessException("登录异常,小程序openId为空");
        }
        if (StrUtil.isBlank(data.getSessionKey())) {
            throw new BusinessException("登录异常,sessionkey值为空");
        }
        //获取token
        String accessToken = miniProgramService.getAccessToken();

        WxUserVo res = this.booleanWxUserByOpenId(data.getOpenid()).getData();
        if (res == null || StrUtil.isBlank(res.getOpenId())) {
            ResultBean<WxUserVo> saveResult = this.saveWxUser(data.getOpenid(),data.getUnionid());
            if(!saveResult.isSuccess()) {
                throw new BusinessException("登录异常");
            }
        }
        WechatAccesstokenVo wechatAccesstokenVo = new WechatAccesstokenVo();
        wechatAccesstokenVo.setAccessToken(accessToken);
        wechatAccesstokenVo.setOpenId(data.getOpenid());
        wechatAccesstokenVo.setSessionKey(data.getSessionKey());

        return ResultBean.success(wechatAccesstokenVo);
    }

    @Override
    public ResultBean<WxUserVo> saveWxUser(String openId, String unionId) {
        WxUserVo wxUserVo = new WxUserVo();
        WxUser wxUser = new WxUser();
        wxUser.setOpenId(openId);
        wxUser.setLoginDate(LocalDateTime.now());
        wxUser.setCreateBy(openId);
        if(!StringUtils.isEmpty(unionId)) {
            wxUser.setUnionId(unionId);
        }
        //当前只保存id和openid到数据库，其他接口来通过openid来存储完整的微信用户资料
        int insert = wxUserMapper.insert(wxUser);
        if(insert == 0) {
            return ResultBean.fail("用户注册未成功");
        }
        BeanUtils.copyProperties(wxUser, wxUserVo);
        return ResultBean.success(wxUserVo);

    }

    @Override
    public ResultBean<JudegAccesstokenVo> judgeTokenFrush() {
        String key = GlobalConstant.RedisNameSpace.SERVICE_APPLET + "MINI_PROGRAM_ACCESS_TOKEN_" + miniProgramProperties.getAppId();
        JudegAccesstokenVo judegAccesstokenVo = new JudegAccesstokenVo();
        String accessToken = redisRepository.get(key);
        if(StrUtil.isNotBlank(accessToken)) {
            judegAccesstokenVo.setExpire(true);
            judegAccesstokenVo.setAccessToken(accessToken);
            return ResultBean.success(judegAccesstokenVo);
        }else {
            judegAccesstokenVo.setAccessToken(null);
            judegAccesstokenVo.setExpire(false);
            return ResultBean.success(judegAccesstokenVo);
        }
    }

    @Override
    public ResultBean loginOut() {
        String key = GlobalConstant.RedisNameSpace.SERVICE_APPLET + "MINI_PROGRAM_ACCESS_TOKEN_" + miniProgramProperties.getAppId();
        String accessToken = redisRepository.get(key);
        if(StrUtil.isNotBlank(accessToken)) {
            boolean del = redisRepository.del(key);
            if(!del) {
                throw new BusinessException("退出登录失败，请重试");
            }
        }
        return ResultBean.success("退出登录成功");
    }

    @Override
    public WxUser getWxUserInfoByopenId(String openId) {
        WxUser wxUser = wxUserMapper.getWxUserInfoByopenId(openId);
        if(null == wxUser) {
            throw new UnAuthorizedException("cannot find wxUser with openId: " + openId);
        }
        return wxUser;
    }
}
